<?php
namespace OrganizationsManagement\Controller;

use OrganizationsManagement\Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use OrganizationsManagement\Service;

class DeployReport extends Simple
{
    /** @var Service\DeployReport $serviceTable */
    protected $deployReport;

    public function __construct(Service\DeployReport $deployReport){
        $this->deployReport = $deployReport;
    }

    public function getDeploys(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = $request->getFilterCriteria()->getConditions();
            $deploys = $this->deployReport->getDeploys($filters, $page, $pageSize);
            $response->addDataSet(new DataSource\DataSet('getDeploys', $deploys));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getResources(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = $request->getFilterCriteria()->getConditions();
            $deploys = $this->deployReport->getResources($filters, $page, $pageSize);
            $response->addDataSet(new DataSource\DataSet('getResources', $deploys));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getProducts(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = $request->getFilterCriteria()->getConditions();
            $deploys = $this->deployReport->getProducts($filters, $page, $pageSize);
            $response->addDataSet(new DataSource\DataSet('getProducts', $deploys));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getServers(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = $request->getFilterCriteria()->getConditions();
            $deploys = $this->deployReport->getServers($filters, $page, $pageSize);
            $response->addDataSet(new DataSource\DataSet('getServers', $deploys));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    public function getParcNegocio(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $filters = $request->getFilterCriteria()->getConditions();
            $deploys = $this->deployReport->getParcNegocio($filters, $page, $pageSize);
            $response->addDataSet(new DataSource\DataSet('getParcNegocio', $deploys));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getNatuCont(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $natucont = $this->deployReport->getNatuCont();
            $response->addDataSet(new DataSource\DataSet('getNatuCont', $natucont));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    public function getClassificacaoCaso(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $classificacao = $this->deployReport->getClassificacaoCaso($filters);
            $response->addDataSet(new DataSource\DataSet('getClassificacaoCaso', $classificacao));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    public function getServicoProdutoReferencia(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $cdNatucont = Utilities::getValueInFilterDataSourceByName($filters, 'CDNATUCONT', true);
            $nrClassificacaocaso = Utilities::getValueInFilterDataSourceByName($filters, 'NRCLASSIFICACAOCASO', true);
            $classificacao = $this->deployReport->getServicoProdutoReferencia($filters,$cdNatucont,$nrClassificacaocaso);
            $response->addDataSet(new DataSource\DataSet('getServicoProdutoReferencia', $classificacao));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getMoticont(DTO\Request\Filter $request, DTO\Response $response){
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $cdProduto = Utilities::getValueInFilterDataSourceByName($filters, 'CDPRODUTO', true);
            $classificacao = $this->deployReport->getMoticont($filters,$cdProduto);
            $response->addDataSet(new DataSource\DataSet('getMoticont', $classificacao));
        } catch(\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    
    
}